import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.mlab as mlab
import sys
import datetime
import glob
from pathlib import Path  

subfolder = 'chart_binance/'

def processStats(fname, pair, period, df_stats_curr_tmp): 
    df = pd.read_csv(fname, delimiter=',', names=['Timestamp','Open','Close','High','Low','Volume'])
    #df.iloc[6:] = df.iloc[6:].astype(float) #https://stackoverflow.com/questions/36814100/pandas-to-numeric-for-multiple-columns
    df['Close'] = pd.to_numeric(df['Close'], errors='coerce').fillna(0)

    df = df.iloc[1:] #remvoe first row 
    df = df.drop_duplicates(keep=False)

    #http://stackoverflow.com/questions/17322109/get-dataframe-row-count-based-on-conditions
    rets=(df['Close']-df['Close'].shift(1))/df['Close']
    dfrets = rets.to_frame(name='Close')
    positives = dfrets[dfrets.Close > 0].count()
    negatives = dfrets[dfrets.Close < 0].count()
    sk = dfrets.skew()
    m = dfrets.mean()
    sd = dfrets.std()
    kurt = dfrets.kurtosis()
    stdUp1 = m + sd
    stdLow1 = m - sd
    stdUp2 = m + (2 * sd)
    stdLow2 = m - (2 * sd)
    stdUp3 = m + (3 * sd)
    stdLow3 = m - (3 * sd)

    #get frequency of positives vs negatives
    sz = rets.sum()
    freqPos = positives/sz
    freqNeg = negatives/sz

    #fixed notmal 
    normalPercStd1 = 0.682 
    normalPercStd2 = 0.954
    normalPercStd3 = 0.998

    #total number of trading day returns within ranges of standard deviation 1, 2, 3

    #this needs to be improved
    #this could be used http://stackoverflow.com/questions/17682613/how-to-convert-a-pandas-dataframe-subset-of-columns-and-rows-into-a-numpy-array

    actualStd1 = 0.0
    actualStd2 = 0.0
    actualStd3 = 0.0

    for i in range(1,3):
        tot = 0.0 
        if i == 1:
            dfrets1 = dfrets.dropna(subset = ['Close'])
            lst = dfrets1['Close'].values.tolist()
        if i == 2:
            dfrets2 = dfrets.dropna(subset = ['Close'])
            lst = dfrets2['Close'].values.tolist()
        if i == 3:
            dfrets3 = dfrets.dropna(subset = ['Close'])
            lst = dfrets3['Close'].values.tolist()
        for ret in lst:
            if ret >= stdLow1.Close and ret <= stdUp1.Close and i == 1:
                actualStd1 += ret
            if ret >= stdLow2.Close and ret <= stdUp2.Close and i == 2:
                actualStd2 += ret
            if ret >= stdLow3.Close and ret <= stdUp3.Close and i == 3:
                actualStd3 += ret
        

    #normaliszed number of trading returns within standard deviation 1, 2,, 3
    #should be compared against actual above
    normalStd1 =  normalPercStd1 * sz
    normalStd2 =  normalPercStd2 * sz
    normalStd3 =  normalPercStd3 * sz

    actualPer1 = actualStd1/sz
    actualPer2 = actualStd2/sz
    actualPer3 = actualStd3/sz

    ##http://matplotlib.org/1.2.1/examples/pylab_examples/histogram_demo.html


    mu, sigma = 100, 15
    x = lst

    #http://matplotlib.org/1.5.0/examples/pylab_examples/histogram_percent_demo.html
    sym = pair
    bins = 5
    plt.xlabel('Returns')
    plt.title('Returns for ' + sym + ' ' + period)
    plt.grid(True)
    plt.hist(x, bins=5, normed=True, facecolor='blue')
    #https://matplotlib.org/1.2.1/examples/pylab_examples/histogram_demo.html
    #mu, sigma = 100, 15
    #y = mlab.normpdf( bins, mu, sigma)
    #l = plt.plot(bins, y, 'r--', linewidth=1)
    plt.grid(True)
    #plt.hist(x, bins=10, normed=True)
    #plt.savefig(subfolder+sym + '_' + period +'_stats3.pdf')
    plt.savefig(subfolder+sym + '_' + period +'_stats3.png')

    plt.cla()
    #plt.show()

    #http://stackoverflow.com/questions/5214578/python-print-string-to-text-file
    with open(subfolder+sym+'_'+period+"_Stats3.txt", "w") as text_file:
        text_file.write("Std dev: {0}".format(sd))
        text_file.write("Mean: {0}".format(m))
        text_file.write("Kurtosis: {0}".format(sk))
        text_file.write("Skew: {0}".format(kurt))
        text_file.write("# positives: {0}".format(positives))
        text_file.write("# negatives: {0}".format(negatives))
        text_file.write("Size: {0}".format(sz))
        text_file.write("Frequency of Positives: {0}".format(freqPos))
        text_file.write("Frequency of Negatives: {0}".format(freqNeg))
        text_file.write("SD1 Up: {0}".format(stdUp1))
        text_file.write("SD1 Low: {0}".format(stdLow1))
        text_file.write("SD2 Up: {0}".format(stdUp2))
        text_file.write("SD2 Low: {0}".format(stdLow2)) 
        text_file.write("SD3 Up: {0}".format(stdUp3))
        text_file.write("SD3 Low: {0}".format(stdLow3))   

    df_stats_curr['FXPair'] = pair

    if(period == '1d'): #1M 1d 1h
        df_stats_curr_tmp['NumPositiveReturns_1d'] = positives.values
        df_stats_curr_tmp['StdDev_1d'] = sd.values
        df_stats_curr_tmp['Mean_1d'] = m.values
    if(period == '1h'):
        df_stats_curr_tmp['NumPositiveReturns_1h'] = positives.values
        df_stats_curr_tmp['StdDev_1h'] = sd.values
        df_stats_curr_tmp['Mean_1h'] = m.values
    if(period == '1M'):
        df_stats_curr_tmp['NumPositiveReturns_1M'] = positives.values
        df_stats_curr_tmp['StdDev_1M'] = sd.values
        df_stats_curr_tmp['Mean_1M'] = m.values

    return df_stats_curr_tmp
    
df_stats_curr = pd.DataFrame({'FXPair':[' '],
    'NumPositiveReturns_1d':[0],
    'StdDev_1d':[0.0],
    'Mean_1d':[0.0],
    'NumPositiveReturns_1h':[0],
    'StdDev_1h':[0.0],
    'Mean_1h':[0.0],
    'NumPositiveReturns_1M':[0],
    'StdDev_1M':[0.0],
    'Mean_1M':[0.0]
    })

df_stats = pd.DataFrame({'FXPair':[' '],
    'NumPositiveReturns_1d':[0],
    'StdDev_1d':[0.0],
    'Mean_1d':[0.0],
    'NumPositiveReturns_1H':[0],
    'StdDev_1h':[0.0],
    'Mean_1h':[0.0],
    'NumPositiveReturns_1M':[0],
    'StdDev_1M':[0.0],
    'Mean_1M':[0.0]
    })


filenames = glob.glob('data/binance*.csv')

for fn in filenames:
    print('reading '+fn)
    data = pd.read_csv(fn, names=['Timestamp', 'Open', 'High', 'Low', 'Close', 'Volume'])

    data = data.iloc[1:] #remvoe first row 


    #data = data[['open','high','low','close','vol']]

    data = data.drop_duplicates(keep=False)

    fname = Path(fn).name
    tokens = fname.split('-')
    sym = tokens[1]
    ext = tokens[2] #extension part of filename 
    tokens2 = ext.split('.')
    tf = tokens2[0] #timeframe 1h, 1M, 1y
    df_stats_curr2 = processStats(fn,sym, tf,df_stats_curr)
    df_stats = pd.concat([df_stats,df_stats_curr2])


# with open('CurrencyPairMajors2.txt') as f:
#     content = f.readlines()
#     content = [x.strip() for x in content]
#     print(content)
#     for n in content:
#         df_stats_curr2 = processStats('/Users//quantlabsnet/csvs/'+n+'Daily.csv',n, 'D',df_stats_curr)
#         df_stats = pd.concat([df_stats,df_stats_curr2])
#         df_stats_curr3 = processStats('/Users//quantlabsnet/csvs/'+n+'4Hr.csv', n, '4H',df_stats_curr2)
#         df_stats = pd.concat([df_stats,df_stats_curr3])
#         df_stats_curr4 = processStats('/Users//quantlabsnet/csvs/'+n+'1Hr.csv', n, '1H',df_stats_curr3) 
#         ç

df_stats = df_stats.loc[df_stats.duplicated(['FXPair'],keep='last'), :]
df_stats = df_stats.loc[df_stats.duplicated(['FXPair'],keep='last'), :]
print(df_stats)
df_stats.to_csv('cryptp_stats.csv')
df_stats.to_excel('crypto_stats.xls')



